package net.sergeych.parsec3

import io.ktor.server.application.*
import io.ktor.server.routing.*
import io.ktor.server.websocket.*
import io.ktor.websocket.*
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.warning
import java.time.Duration

/**
 * Creates a ktor server initialization module capable to perform p3 transport layer (not secure).
 * It could be used as is or as transport for p3.1
 */
fun <S: WithAdapter, H : CommandHost<S>>Application.parsec3TransportServer(
    api: H,
    exceptionsRegistry: ExceptionsRegistry = ExceptionsRegistry(),
    path: String = "/api/p3",
    f: AdapterBuilder<S, H>.() -> Unit,
) {
    val log = LogTag("P3WST")
    try {
        install(WebSockets) {
            pingPeriod = Duration.ofSeconds(45)
            timeout = Duration.ofSeconds(15)
            maxFrameSize = Long.MAX_VALUE
            masking = false
        }
    }
    catch(_: DuplicatePluginException) {
        // webspkects are already initialized, it is OK
    }

    val builder = AdapterBuilder(api, exceptionsRegistry, f)

    routing {
        webSocket(path) {
            val adapter = builder.create { outgoing.send(Frame.Binary(true, it)) }
            adapter.onCancel = {close()}
            for (frame in (this@webSocket).incoming) {
                when (frame) {
                    is Frame.Binary -> {
                        adapter.receiveFrame(frame.readBytes())
                    }
                    else -> {
                        log.warning { "unsupported frame type: $frame" }
                    }
                }
            }
            adapter.close()
        }

    }
}

//fun <S: WithAdapter, H : CommandHost<S>>Application.parsec3SecureServer(
//    api: H,
//    exceptionsRegistry: ExceptionsRegistry = ExceptionsRegistry(),
//    path: String = "/api/p3",
//    f: AdapterBuilder<S, H>.() -> Unit,
//) {
//    val log = LogTag("P3WSS")
////    parsec3TransportServer(api, ex)
////    install(WebSockets) {
////        pingPeriod = Duration.ofSeconds(45)
////        timeout = Duration.ofSeconds(15)
////        maxFrameSize = Long.MAX_VALUE
////        masking = false
////    }
//    val dh = DiffieHellman()
//}
