package net.sergeych.parsec3

import org.mapdb.DB
import org.mapdb.DBMaker
import org.mapdb.Serializer
import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.Paths
import java.util.concurrent.ConcurrentMap

actual fun defaultNamedStorage(name: String): KVStorage {
    val path = Paths.get(System.getProperty("user.home"), ".kvstorages")
    Files.createDirectories(path)
    return MapdbBinaryKVStorage("$path/$name.kvs")
}

class MapdbBinaryKVStorage(dbFileName: String): KVStorage {

    private val db: DB
    private val map: ConcurrentMap<String, ByteArray>

    init {
        db = DBMaker.fileDB(dbFileName)
            .fileMmapEnableIfSupported()
            .closeOnJvmShutdown()
            .transactionEnable()
            .make()
        map = db.hashMap("kvBinaryStorage", Serializer.STRING, Serializer.BYTE_ARRAY)
            .createOrOpen()
    }

    override val keys: Set<String>
        get() = map.keys

    override fun get(key: String): ByteArray? = map[key]

    override fun set(key: String, value: ByteArray?) {
        if( value != null)
            map[key] = value
        else
            map.remove(key)
        db.commit()
    }

    override fun contains(key: String): Boolean = key in map
}
